@@ -1,4 +1,4 @@
-# $Id: ODBC.pm 14741 2011-03-06 17:11:57Z mjevans $
+# $Id: ODBC.pm 14890 2011-06-22 19:33:10Z mjevans $
 #
 # Copyright (c) 1994,1995,1996,1998  Tim Bunce
 # portions Copyright (c) 1997-2004  Jeff Urlwin
@@ -10,7 +10,7 @@
 
 ## no critic (ProhibitManyArgs ProhibitMultiplePackages)
 
-require 5.006;
+require 5.008;
 
 # NOTE: Don't forget to update the version reference in the POD below too.
 # NOTE: If you create a developer release x.y_z ensure y is greater than
@@ -19,7 +19,7 @@ require 5.006;
 # see discussion on dbi-users at
 # http://www.nntp.perl.org/group/perl.dbi.dev/2010/07/msg6096.html and
 # http://www.dagolden.com/index.php/369/version-numbers-should-be-boring/
-$DBD::ODBC::VERSION = '1.29';
+$DBD::ODBC::VERSION = '1.31';
 
 {
     ## no critic (ProhibitMagicNumbers ProhibitExplicitISA)
@@ -32,7 +32,7 @@ $DBD::ODBC::VERSION = '1.29';
 
     @ISA = qw(Exporter DynaLoader);
 
-    # my $Revision = substr(q$Id: ODBC.pm 14741 2011-03-06 17:11:57Z mjevans $, 13,2);
+    # my $Revision = substr(q$Id: ODBC.pm 14890 2011-06-22 19:33:10Z mjevans $, 13,2);
 
     require_version DBI 1.21;
 
@@ -129,23 +129,25 @@ $DBD::ODBC::VERSION = '1.29';
 
     sub private_attribute_info {
         return {
-                odbc_ignore_named_placeholders => undef, # sth and dbh
-                odbc_default_bind_type => undef, # sth and dbh
-                odbc_force_bind_type => undef, # sth and dbh
-                odbc_force_rebind => undef, # sth and dbh
-                odbc_async_exec => undef, # sth and dbh
-                odbc_exec_direct => undef,
-                odbc_SQL_ROWSET_SIZE => undef,
-                odbc_SQL_DRIVER_ODBC_VER => undef,
-                odbc_cursortype => undef,
-                odbc_query_timeout => undef, # sth and dbh
-                odbc_has_unicode => undef,
-                odbc_out_connect_string => undef,
-                odbc_version => undef,
-                odbc_err_handler => undef,
-                odbc_putdata_start => undef, # sth and dbh
-                odbc_column_display_size => undef, # sth and dbh
-                odbc_utf8_on => undef # sth and dbh
+            odbc_ignore_named_placeholders => undef, # sth and dbh
+            odbc_default_bind_type         => undef, # sth and dbh
+            odbc_force_bind_type           => undef, # sth and dbh
+            odbc_force_rebind              => undef, # sth and dbh
+            odbc_async_exec                => undef, # sth and dbh
+            odbc_exec_direct               => undef,
+            odbc_old_unicode               => undef,
+            odbc_describe_parameters       => undef,
+            odbc_SQL_ROWSET_SIZE           => undef,
+            odbc_SQL_DRIVER_ODBC_VER       => undef,
+            odbc_cursortype                => undef,
+            odbc_query_timeout             => undef, # sth and dbh
+            odbc_has_unicode               => undef,
+            odbc_out_connect_string        => undef,
+            odbc_version                   => undef,
+            odbc_err_handler               => undef,
+            odbc_putdata_start             => undef, # sth and dbh
+            odbc_column_display_size       => undef, # sth and dbh
+            odbc_utf8_on                   => undef # sth and dbh
                };
     }
 
@@ -490,16 +492,19 @@ $DBD::ODBC::VERSION = '1.29';
 
     sub private_attribute_info {
         return {
-                odbc_ignore_named_placeholders => undef, # sth and dbh
-                odbc_default_bind_type => undef, # sth and dbh
-                odbc_force_bind_type => undef, # sth and dbh
-                odbc_force_rebind => undef, # sth and dbh
-                odbc_async_exec => undef, # sth and dbh
-                odbc_query_timeout => undef, # sth and dbh
-                odbc_putdata_start => undef, # sth and dbh
-                odbc_column_display_size => undef, # sth and dbh
-                odbc_utf8_on => undef # sth and dbh
-               };
+            odbc_ignore_named_placeholders => undef, # sth and dbh
+            odbc_default_bind_type         => undef, # sth and dbh
+            odbc_force_bind_type           => undef, # sth and dbh
+            odbc_force_rebind              => undef, # sth and dbh
+            odbc_async_exec                => undef, # sth and dbh
+            odbc_query_timeout             => undef, # sth and dbh
+            odbc_putdata_start             => undef, # sth and dbh
+            odbc_column_display_size       => undef, # sth and dbh
+            odbc_utf8_on                   => undef, # sth and dbh
+            odbc_exec_direct               => undef, # sth and dbh
+            odbc_old_unicode               => undef, # sth and dbh
+            odbc_describe_parameters => undef, # sth and dbh
+        };
     }
 
     sub ColAttributes { # maps to SQLColAttributes
@@ -562,7 +567,7 @@ DBD::ODBC - ODBC Driver for DBI
 
 =head1 VERSION
 
-This documentation refers to DBD::ODBC version 1.29.
+This documentation refers to DBD::ODBC version 1.31.
 
 =head1 SYNOPSIS
 
@@ -866,22 +871,84 @@ Do not confuse this with DBD::ODBC's unicode support. The
 C<odbc_utf8_on> attribute only applies to non-unicode enabled builds
 of DBD::ODBC.
 
+=head3 odbc_old_unicode
+
+Defaults to off. If set to true returns DBD::ODBC to the old unicode
+behavior in 1.29 and earlier. You can also set this on the prepare
+method.
+
+By default DBD::ODBC now binds all char columns as SQL_WCHARs meaning
+the driver is asked to return the bound data as wide (Unicode)
+characters encoded in UCS2. So long as the driver supports the ODBC
+Unicode API properly this should mean you get your data back correctly
+in Perl even if it is in a character set (codepage) different from the
+one you are working in.
+
+However, if you wrote code using DBD::ODBC 1.29 or earlier and knew
+DBD::ODBC bound varchar/longvarchar columns as SQL_CHARs and decoded
+them yourself the new behaviour will adversely affect you (sorry). To
+revret to the old behaviour set odbc_old_unicode to true.
+
+You can also set this attribute in the attributes passed to the
+prepare method.
+
+See the stackoverflow question at
+L<http://stackoverflow.com/questions/5912082>, the RT at
+L<http://rt.cpan.org/Public/Bug/Display.html?id=67994> and lastly a
+small discussion on dbi-dev at
+L<http://www.nntp.perl.org/group/perl.dbi.dev/2011/05/msg6559.html>.
+
+=head3 odbc_describe_parameters
+
+Defaults to on. When set this allows DBD::ODBC to call SQLDescribeParam
+(if the driver supports it) to retrieve information about any
+parameters.
+
+When off/false DBD::ODBC will not call SQLDescribeParam and defaults
+to binding parameters as SQL_CHAR/SQL_WCHAR depending on the build
+type.
+
+You do not have to disable odbc_describe_parameters just because your
+driver does not support SQLDescribeParam as DBD::ODBC will work this
+out at the start via SQLGetFunctions.
+
+Note: disabling odbc_describe_parameters when your driver does support
+SQLDescribeParam may prevent DBD::ODBC binding parameters for some
+column types properly.
+
+You can also set this attribute in the attributes passed to the
+prepare method.
+
+This attribute was added so someone moving from freeTDS (a driver
+which does not support SQLDescribeParam) to a driver which does
+support SQLDescribeParam could do so without changing any Perl. The
+situation was very specific since dates were being bound as dates when
+SQLDescribeParam was called and chars without and the data format was
+not a supported date format.
+
 =head2 Private connection attributes
 
 =head3 odbc_err_handler
 
-B<NOTE:> There should be no reason to use this now as there is a DBI
-attribute of a similar name. In future versions this attribute will
-be deleted.
+B<NOTE:> You might want to look at DBI's error handler before using
+the one in DBD::ODBC however, there are subtle
+differences. DBD::ODBC's odbc_err_handler is called for error B<and>
+informational diagnostics i.e., it is called when an ODBC call fails
+the SQL_SUCCEEDED macro which means the ODBC call returned SQL_ERROR
+(-1) or SQL_SUCCESS_WITH_INFO (1).
 
-Allow errors to be handled by the application.  A call-back function
-supplied by the application to handle or ignore messages.
+Allow error and informational diagnostics to be handled by the
+application.  A call-back function supplied by the application to
+handle or ignore messages.
 
-The callback function receives three parameters: state (string),
-error (string) and the native error code (number).
+The callback function receives four parameters: state (string),
+error (string), native error code (number) and the status returned
+from the last ODBC API. The fourth argument was added in 1.30_7.
 
 If the error handler returns 0, the error is ignored, otherwise the
-error is passed through the normal DBI error handling.
+error is passed through the normal DBI error handling. Note, if the
+status is SQL_SUCCESS_WITH_INFO this will B<not> reach the DBI error
+handler as it is not an error.
 
 This can also be used for procedures under MS SQL Server (Sybase too,
 probably) to obtain messages from system procedures such as DBCC.
@@ -889,13 +956,13 @@ Check F<t/20SQLServer.t> and F<t/10handler.t>.
 
   $dbh->{RaiseError} = 1;
   sub err_handler {
-     ($state, $msg, $native) = @_;
+     ($state, $msg, $native, $rc, $status) = @_;
      if ($state = '12345')
          return 0; # ignore this error
      else
          return 1; # propagate error
   }
-  $dbh->{odbc_err_handler} = \$err_handler;
+  $dbh->{odbc_err_handler} = \&err_handler;
   # do something to cause an error
   $dbh->{odbc_err_handler} = undef; # cancel the handler
 
@@ -1129,7 +1196,7 @@ C<$length> so DBD::ODBC grows it appropriately.
 
 You can retrieve a lob in chunks like this:
 
-  $sth->bind_col($column, undef, {BindAsLOB=>1});
+  $sth->bind_col($column, undef, {TreatAsLOB=>1});
   while(my $retrieved = $sth->odbc_lob_read($column, \my $data, $length)) {
       print "retrieved=$retrieved lob_data=$data\n";
   }
@@ -1307,7 +1374,7 @@ You call SQLSpecialColumns like this:
 
 Handled as of version 0.28
 
-head3 ColAttributes
+=head3 ColAttributes
 
 B<This private function is now superceded by DBI's statement attributes
 NAME, TYPE, PRECISION, SCALE, NULLABLE etc).>
@@ -1342,7 +1409,7 @@ returns strange values for column name e.g., '20291'. It is wiser to
 use DBI's NAME and NAME_xx attributes for portability.
 
 
-head3 DescribeCol
+=head3 DescribeCol
 
 B<This private function is now superceded by DBI's statement attributes
 NAME, TYPE, PRECISION, SCLARE, NULLABLE etc).>
@@ -1355,6 +1422,46 @@ You call SQLDescribeCol like this:
 The returned array contains the column attributes in the order described
 in the ODBC specification for SQLDescribeCol.
 
+=head2 Additional bind_col attributes
+
+DBD::ODBC supports a few additional attributes which may be passed to
+the bind_col method in the attributes.
+
+=head3 DiscardString
+
+See DBI's sql_type_cast utility function.
+
+If you bind a column as a specific type (SQL_INTEGER, SQL_DOUBLE and
+SQL_NUMERIC are the only ones supported currently) and you add
+DiscardString to the prepare attributes then if the returned bound
+data is capable of being converted to that type the scalar's pv (the
+string portion of a scalar) is cleared.
+
+This is especially useful if you are using a module which uses a
+scalars flags and/or pv to decide if a scalar is a number. JSON::XS
+does this and without this flag you have to add 0 to all bound column
+data returning numbers to get JSON::XS to encode it is N instead of
+"N".
+
+=head3 StrictlyTyped
+
+See DBI's sql_type_cast utility function.
+
+See L</DiscardString> above.
+
+Specifies that when DBI's sql_type_cast function is called on returned
+data where a bind type is specified that if the conversion cannot be
+performed an error will be raised.
+
+This is probably not a lot of use with DBD::ODBC as if you ask for say
+an SQL_INTEGER and the data is not able to be converted to an integer
+the ODBC driver will problably return "Invalid character value for
+cast specification (SQL-22018)".
+
+=head3 TreatAsLOB
+
+See L</odbc_lob_read>.
+
 =head2 Tracing
 
 DBD::ODBC now supports the parse_trace_flag and parse_trace_flags
@@ -1443,7 +1550,7 @@ believes to be a placeholder in a comment e.g.,
 I cannot be exact about support for ignoring placeholders in literals
 but it has existed for a long time in DBD::ODBC. Support for ignoring
 placeholders in comments was added in 1.24_2. If you find a case where
-a named placeholder is not ignored and should be see
+a named placeholder is not ignored and should be, see
 L</odbc_ignore_named_placeholders> for a workaround and mail me an
 example along with your ODBC driver name.
 
@@ -1537,6 +1644,13 @@ ODBC inherently allows this. Therefore you can do:
   # some DBDs will ignore the type in the following, DBD::ODBC does not
   $sth->bind_param(1, $data, DBI::SQL_VARCHAR);
 
+=head3 disconnect and transactions
+
+DBI does not define whether a driver commits or rolls back any
+outstanding transaction when disconnect is called. As such DBD::ODBC
+cannot deviate from the specification but you should know it rolls
+back an uncommitted transaction when disconnect is called if
+SQLDisconnect returns state 25000 (transaction in progress).
 
 =head2 Unicode
 
@@ -1917,6 +2031,8 @@ L<Test::Simple>
 
 =head1 INCOMPATIBILITIES
 
+None known.
+
 =head1 BUGS AND LIMITATIONS
 
 None known other than the deviations from the DBI specification mentioned
